#
#  Copyright  2015, VMware Inc. All rights reserved.
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of the software in this file (the "Software"), to deal in the Software 
#  without restriction, including without limitation the rights to use, copy, 
#  modify, merge, publish, distribute, sublicense, and/or sell copies of the 
#  Software, and to permit persons to whom the Software is furnished to do so, 
#  subject to the following conditions:
#  
#  The above copyright notice and this permission notice shall be included in 
#  all copies or substantial portions of the Software.
#  
#  The names "VMware" and "VMware, Inc." must not be used to endorse or promote 
#  products derived from the Software without the prior written permission of 
#  VMware, Inc.
#  
#  Products derived from the Software may not be called "VMware", nor may 
#  "VMware" appear in their name, without the prior written permission of 
#  VMware, Inc.
#  
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
#  VMWARE,INC. BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
#  IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
#  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#


<#
    .SYNOPSIS
     Sample Powershell script to deploy a VMware Access Point virtual appliance using ovftool.
    .EXAMPLE
     .\apdeploy.ps1 -iniFile ap1.ini 
#>

param([string]$iniFile = "ap.ini")

#
# Function to parse token valuse from a .ini configuration file
#

function Import-ini {
	param ($file)

	$ini = @{}
	switch -regex -file $file
	{
    		"^\[(.+)\]$" {
        		$section = $matches[1]
        		$ini[$section] = @{}
    		}
    		"(.+)=(.+)" {
        		$name,$value = $matches[1..2]
        		$ini[$section][$name] = $value
    		}
	}
	$ini
}

#
# Function to write an error message in red with black background
#

function Write-error-string {
	param ($string)
	write-host $string -foregroundcolor red -backgroundcolor black
}

#
# Function to decrypt an encrypted password
#

function ConvertFrom-SecureToPlain {
    
    param( [Parameter(Mandatory=$true)][System.Security.SecureString] $SecurePassword)
    
    # Create a "password pointer".
    $PasswordPointer = [Runtime.InteropServices.Marshal]::SecureStringToBSTR($SecurePassword)
    
    # Get the plain text version of the password.
    $PlainTextPassword = [Runtime.InteropServices.Marshal]::PtrToStringAuto($PasswordPointer)
    
    # Free the pointer.
    [Runtime.InteropServices.Marshal]::ZeroFreeBSTR($PasswordPointer)
    
    # Return the plain text password.
    $PlainTextPassword
    
}

#
# Function to prompt the user for an Access Point VM name and validate the input
#

function GetAPName {
	$valid=0
	while (! $valid) {
		$apName = Read-host "Enter a name for this appliance"
		if (($apName.length -lt 1) -Or ($apName.length -gt 32)) { 
			Write-error-string "Error: Virtual machine name must be between 1 and 32 characters in length"
		} else {
			$valid=1
		}
	}
	$apName 
}

#
# Function to prompt the user for an Access Point root password validate the input
#

function GetRootPwd {
	param( [Parameter(Mandatory=$true)] $apName)
	$match=0
	while (! $match) {
		$valid=0
		while (! $valid) { 
			$rootPwd = Read-Host -assecurestring "Enter a root password for" $apName
			$rootPwd = ConvertFrom-SecureToPlain $rootPwd
			if ($rootPwd.length -lt 6) {
				Write-error-string "Error: Password must contain at least 6 characters"
				Continue
			}
			$valid=1
		}

		$rootPwd2 = Read-Host -assecurestring "Re-enter the root password"
		$rootPwd2 = ConvertFrom-SecureToPlain $rootPwd2
		if ($rootPwd -ne $rootPwd2) {
			Write-error-string "Error: re-entered password does not match"
		} else {
			$match=1
		}
	}
	$rootPwd = $rootPwd -replace '"', '\"'
	$rootPwd = $rootPwd -replace "'", "\047"
	$rootPwd
}

#
# Function to prompt the user for an Access Point admin password validate the input
#

function GetAdminPwd {
	param( [Parameter(Mandatory=$true)] $apName)
	$match=0
	while (! $match) {
		$valid=0
		while (! $valid) { 
			$adminPwd = Read-Host -assecurestring "Enter an optional admin password for the REST API management access for" $apName
			$adminPwd = ConvertFrom-SecureToPlain $adminPwd
			if ($adminPwd.length -eq 0) {
				return
			}

			if ($adminPwd.length -lt 8) {
				Write-error-string "Error: Password must contain at least 8 characters"
				Write-error-string "Password must contain at least 8 characters including an upper case letter, a lower case letter, a digit and a special character from !@#$%*()"
				Continue
			}
			if (([regex]"[0-9]").Matches($adminPwd).Count -lt 1 ) {
				Write-error-string "Error: Password must contain at least 1 numeric digit"
				Write-error-string "Password must contain at least 8 characters including an upper case letter, a lower case letter, a digit and a special character from !@#$%*()"
				Continue
			}
			if (([regex]"[A-Z]").Matches($adminPwd).Count -lt 1 ) {
				Write-error-string "Error: Password must contain at least 1 upper case character (A-Z)"
				Write-error-string "Password must contain at least 8 characters including an upper case letter, a lower case letter, a digit and a special character from !@#$%*()"
				Continue
			}
			if (([regex]"[a-z]").Matches($adminPwd).Count -lt 1 ) {
				Write-error-string "Error: Password must contain at least 1 lower case character (a-z)"
				Write-error-string "Password must contain at least 8 characters including an upper case letter, a lower case letter, a digit and a special character from !@#$%*()"
				Continue
			}
			if (([regex]"[!@#$%*()]").Matches($adminPwd).Count -lt 1 ) {
				Write-error-string "Error: Password must contain at least 1 special character (!@#$%*())"
				Write-error-string "Password must contain at least 8 characters including an upper case letter, a lower case letter, a digit and a special character from !@#$%*()"
				Continue
			}
			$valid = 1
		}

		$adminPwd2 = Read-Host -assecurestring "Re-enter the admin password"
		$adminPwd2 = ConvertFrom-SecureToPlain $adminPwd2
		if ($adminPwd -ne $adminPwd2) {
			Write-error-string "Error: re-entered password does not match"
		} else {
			$match=1
		}
	}
	$adminPwd = $adminPwd -replace '"', '\"'
	$adminPwd = $adminPwd -replace "'", "\047"
	$adminPwd
}

Write-host Access Point virtual appliance deployment script


if (!(Test-path $iniFile)) {
	write-error-string "Error: Configuration file ($iniFile) not found."
	Exit
}



$settings = Import-ini $iniFile

$sslCertsFile=$settings.SSLcert.pemCerts

if ($sslCertsFile.length -gt 0) {

	if (!(Test-path $sslCertsFile)) {
		write-error-string "Error: PEM Certificate file not found ($sslCertsFile)"
		Exit
	}

	$rsaPrivKeyFile=$settings.SSLcert.pemPrivKey

	if ($rsaPrivKeyFile.length -eq 0) {
		write-error-string "Error: PEM RSA private key file pemPrivKey not specified"
		Exit
	}

	if (!(Test-path $rsaPrivKeyFile)) {
		write-error-string "Error: PEM RSA private key file not found ($rsaPrivKeyFile)"
		Exit
	}

	$sslcerts = (Get-Content $sslCertsFile | Out-String) -replace "`r`n", "\\n" 
	$rsaprivkey = (Get-Content $rsaPrivKeyFile | Out-String) -replace "`r`n", "\\n"

	if (
$rsaprivkey -like "*-----BEGIN RSA PRIVATE KEY-----*") {
		Write-host Deployment will use the specified SSL/TLS server certificate
	} else {
		write-error-string "Error: Invalid private key PEM file (pemPrivKey) specified. It must contain an RSA private key."
		Exit
	}
} else {
	Write-host Deployment will use a self-signed SSL/TLS server certificate
}

$apName=$settings.General.name

$logfile="log-$apName.txt"
Remove-item -path $logfile -ErrorAction SilentlyContinue

$ds=$settings.General.ds

if (!$ds) {
	write-error-string "Error: ds in the [General] section of $iniFile is missing. Set ds= followed by the data store name."
	Exit
}

$proxyDestinationUrl=$settings.Horizon.proxyDestinationUrl

if ($proxyDestinationUrl.length -eq 0) {
	write-error-string "Error: proxyDestinationUrl in the [Horizon] section of $iniFile is missing."
	Exit
}

#
# Assign and validate network settings
#

$dns=$settings.General.dns
$netInternet=$settings.General.netInternet
$ip0=$settings.General.ip0

$netManagementNetwork=$settings.General.netManagementNetwork
$ip1=$settings.General.ip1

$netBackendNetwork=$settings.General.netBackendNetwork
$ip2=$settings.General.ip2

if ((!$ip0) -And (!$ip1) -And (!$ip2)) {

	#
	# No IP addresses specified so we will use DHCP for address allocation
	#

	$ipAllocationPolicy = "dhcpPolicy"

} else {

	$ipAllocationPolicy = "fixedPolicy"

}

$deploymentOption=$settings.General.deploymentOption

if (!$deploymentOption) {
	$deploymentOption="onenic"
}

if ($ipAllocationPolicy -eq "fixedPolicy") {
	switch ($deploymentOption) {

		onenic {
			if (!$ip0) {
				write-error-string "Error: ip0 in the [General] section of $iniFile is missing."
				Exit
			}
		}
		twonic {
			if (!$ip0) {
				write-error-string "Error: ip0 in the [General] section of $iniFile is missing."
				Exit
			}
			if (!$ip1) {
				write-error-string "Error: ip1 in the [General] section of $iniFile is missing."
				Exit
			}
		}
		threenic {
			if (!$ip0) {
				write-error-string "Error: ip0 in the [General] section of $iniFile is missing."
				Exit
			}
			if (!$ip1) {
				write-error-string "Error: ip1 in the [General] section of $iniFile is missing."
				Exit
			}
			if (!$ip2) {
				write-error-string "Error: ip2 in the [General] section of $iniFile is missing."
				Exit
			}
		}
		default {
			write-error-string "Error: deploymentOption=$deploymentOption in the [General] section of $iniFile is not valid."
			write-error-string "It must be set to onenic, twonic or threenic."
			Exit

		}
	}
}

$proxyDestinationUrlThumbprints=$settings.Horizon.proxyDestinationUrlThumbprints  -replace ":","="

$blastExternalUrl=$settings.Horizon.blastExternalUrl

$pcoipExternalUrl=$settings.Horizon.pcoipExternalUrl
if ($pcoipExternalUrl.length -gt 0) {
	if (([regex]"[.]").Matches($pcoipExternalUrl).Count -ne 3 ) {
		write-error-string "Error: Invalid pcoipExternalUrl value specified ($pcoipExternalUrl). It must contain an IPv4 address."
		Exit
	}
}

$tunnelExternalUrl=$settings.Horizon.tunnelExternalUrl
$syslogUrl=$settings.General.syslogUrl

$ovftool = "C:\Program Files\VMware\VMware OVF Tool\ovftool.exe"

if (!(Test-path $ovftool)) {
	write-error-string "Error: ovftool command not found ($ovftool)"
	Exit
}

$source = $settings.General.source
$target = $settings.General.target

if (!(Test-path $source)) {
	write-error-string "Error: Source Accesss Point image not found ($source)"
	Exit
}

if ($apName.length -gt 32) { 
	Write-error-string "Error: Virtual machine name must be no more than 32 characters in length"
	Exit
}

if (!$apName) {
	$apName = GetAPName
}

$rootPwd = GetRootPwd $apName
$adminPwd = GetAdminPwd $apName

$ovfOptions="--X:enableHiddenProperties --X:waitForIp --X:logFile=$logfile --X:logLevel=verbose --powerOffTarget --powerOn --overwrite"

#
# Access Point system settings
#

$systemSettings = "\'systemSettings\':"
$systemSettings += "{"
$systemSettings += "\'locale\': \'en_US\'"
$systemSettings += ", "
$systemSettings += "\'cipherSuites\': \'TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA,TLS_RSA_WITH_AES_128_CBC_SHA,SSL_RSA_WITH_RC4_128_SHA\'"
$systemSettings += ", "
$systemSettings += "\'ssl30Enabled\': false, \'tls10Enabled\': true, \'tls11Enabled\': true, \'tls12Enabled\': true"
$systemSettings += "}"

#
# Access Point Horizon View settings
#

$edgeServiceSettings = "\'edgeServiceSettingsList\':"
$edgeServiceSettings += "{ \'edgeServiceSettingsList\': ["
$edgeServiceSettings += "{ \'identifier\': \'VIEW\'"
$edgeServiceSettings += ","
$edgeServiceSettings += "\'enabled\': true"
$edgeServiceSettings += ","
$edgeServiceSettings += "\'proxyDestinationUrl\': \'"+$proxyDestinationUrl+"\'"
if ($proxyDestinationUrlThumbprints.length -gt 0) {
	$edgeServiceSettings += ","
	$edgeServiceSettings += "\'proxyDestinationUrlThumbprints\': \'"+$proxyDestinationUrlThumbprints+"\'"
}

if ($pcoipExternalUrl.length -gt 0) {
	$edgeServiceSettings += ","
	$edgeServiceSettings += "\'pcoipEnabled\':true"
	$edgeServiceSettings += ","
	$edgeServiceSettings += "\'pcoipExternalUrl\': \'"+$pcoipExternalUrl+"\'"
} else {
	$edgeServiceSettings += ","
	$edgeServiceSettings += "\'pcoipEnabled\':false"
}

if ($blastExternalUrl.length -gt 0) {
	$edgeServiceSettings += ","
	$edgeServiceSettings += "\'blastEnabled\':true"
	$edgeServiceSettings += ","
	$edgeServiceSettings += "\'blastExternalUrl\': \'"+$blastExternalUrl+"\'"
} else {
	$edgeServiceSettings += ","
	$edgeServiceSettings += "\'blastEnabled\':false"
}

if ($tunnelExternalUrl.length -gt 0) {
	$edgeServiceSettings += ","
	$edgeServiceSettings += "\'tunnelEnabled\':true"
	$edgeServiceSettings += ","
	$edgeServiceSettings += "\'tunnelExternalUrl\': \'"+$tunnelExternalUrl+"\'"
} else {
	$edgeServiceSettings += ","
	$edgeServiceSettings += "\'tunnelEnabled\':false"
}

$edgeServiceSettings += ","
$edgeServiceSettings += "\'proxyPattern\':\'/\'"
$edgeServiceSettings += "}"
$edgeServiceSettings += "] }"

if ($sslcerts.length -gt 0) {
	$cert="\'certificateWrapper\': { \'privateKeyPem\': \'"
	$cert+=$rsaprivkey
	$cert+="\', \'certChainPem\': \'"
	$cert+=$sslcerts
	$cert+="\' }"
	$jsonString = "prop:settingsJSON={"+$cert+", "+$edgeServiceSettings+", "+$systemSettings+"}"
} else {
	$jsonString = "prop:settingsJSON={"+$edgeServiceSettings+", "+$systemSettings+"}"
}

$configfile = "${env:APPDATA}\VMware\ovftool.cfg"
$jsonString = $jsonString -replace "'", '"'
[IO.File]::WriteAllLines($configfile, $jsonString)

#
# ESX datastore name
#

$ovfOptions += " -ds='$ds'"

$ovfOptions += " --name='"+$apName+"'"
$ovfOptions += " --prop:rootPassword='"+$rootPwd+"'"
if ($adminPwd.length -gt 0) {
	$ovfOptions += " --prop:adminPassword='"+$adminPwd+"'"
}
if ($syslogUrl.length -gt 0) {
	$ovfOptions += " --prop:syslogUrl='"+$syslogUrl+"'"
}

$ovfOptions += " --ipAllocationPolicy=$ipAllocationPolicy"
$ovfOptions += " --deploymentOption=$deploymentOption"

if ($dns.length -gt 0) {
	$ovfOptions += " --prop:DNS=$dns"
}

if ($ip0.length -gt 0) {
	$ovfOptions += " --prop:ip0=$ip0"
}

if ($ip1.length -gt 0) {
	$ovfOptions += " --prop:ip1=$ip1"
}

if ($ip2.length -gt 0) {
	$ovfOptions += " --prop:ip2=$ip2"
}

if ($netInternet.length -gt 0) {
	$ovfOptions += " --net:Internet='"+$netInternet+"'"
}

if ($netManagementNetwork.length -gt 0) {
	$ovfOptions += " --net:ManagementNetwork='"+$netManagementNetwork+"'"
}

if ($netBackendNetwork.length -gt 0) {
	$ovfOptions += " --net:BackendNetwork='"+$netBackendNetwork+"'"
}

$ovfOptions = $ovfOptions -replace "'", '"'
$ovfOptions = $ovfOptions -replace "\\047", "'"
$ovfOptions = "$ovfOptions".Split(" ")

#write-host $ovfOptions

& $ovftool $ovfOptions $source $target

if ($? -eq "0") {
	if ($ipAllocationPolicy -eq "fixedPolicy") {
		Write-host "Note that the IP addresses will be set to the specified IP addresses for each NIC"
	}
	Write-host "Access Point virtual appliance $apName deployed successfully"
} else {
	Write-host "Access Point deployment failed. Further information may be found in the log file $logfile"
}

Remove-item -path $configfile